/*
 * Copyright 2016 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.kie.workbench.common.screens.library.client.screens;

import javax.inject.Inject;

import com.google.gwt.event.dom.client.ClickEvent;
import org.jboss.errai.common.client.dom.Button;
import org.jboss.errai.common.client.dom.Div;
import org.jboss.errai.common.client.dom.HTMLElement;
import org.jboss.errai.ui.client.local.api.IsElement;
import org.jboss.errai.ui.client.local.spi.TranslationService;
import org.jboss.errai.ui.shared.api.annotations.DataField;
import org.jboss.errai.ui.shared.api.annotations.EventHandler;
import org.jboss.errai.ui.shared.api.annotations.Templated;

@Templated
public class EmptyLibraryView implements EmptyLibraryScreen.View,
                                         IsElement {

    private EmptyLibraryScreen presenter;

    @Inject
    private TranslationService ts;

    @Inject
    @DataField("main-actions")
    Div mainActions;

    @Inject
    @DataField("try-samples")
    Button trySamples;

    @Inject
    @DataField("import-project")
    Button importProject;

    @Override
    public void init(final EmptyLibraryScreen presenter) {
        this.presenter = presenter;

        final boolean userCanCreateProjects = presenter.userCanCreateProjects();
        trySamples.setHidden(!userCanCreateProjects);
        importProject.setHidden(!userCanCreateProjects);
    }

    @Override
    public void addAction(final HTMLElement action) {
        mainActions.appendChild(action);
    }

    @EventHandler("try-samples")
    public void trySamples(final ClickEvent event) {
        presenter.trySamples();
    }

    @EventHandler("import-project")
    public void importProject(final ClickEvent event) {
        presenter.importProject();
    }
}