/*
 * Copyright 2016 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.kie.workbench.common.screens.server.management.client.events;

import org.kie.server.controller.api.model.spec.ServerTemplateKey;

public class ServerTemplateSelected {

    private final ServerTemplateKey serverTemplateKey;
    private final String containerId;

    public ServerTemplateSelected( final ServerTemplateKey serverTemplateKey ) {
        this( serverTemplateKey, null );
    }

    public ServerTemplateSelected( final ServerTemplateKey serverTemplateKey,
                                   final String containerId ) {
        this.serverTemplateKey = serverTemplateKey;
        this.containerId = containerId;
    }

    public ServerTemplateKey getServerTemplateKey() {
        return serverTemplateKey;
    }

    public String getContainerId() {
        return containerId;
    }

    @Override
    public boolean equals( final Object o ) {
        if ( this == o ) {
            return true;
        }
        if ( !( o instanceof ServerTemplateSelected ) ) {
            return false;
        }

        final ServerTemplateSelected that = (ServerTemplateSelected) o;

        if ( serverTemplateKey != null ? !serverTemplateKey.equals( that.serverTemplateKey ) : that.serverTemplateKey != null ) {
            return false;
        }
        return containerId != null ? containerId.equals( that.containerId ) : that.containerId == null;

    }

    @Override
    public int hashCode() {
        int result = serverTemplateKey != null ? serverTemplateKey.hashCode() : 0;
        result = 31 * result + ( containerId != null ? containerId.hashCode() : 0 );
        return result;
    }
}
